package PersonLib;

import java.util.*;

public class Person implements Comparable<Person>
{
	private String name; 
	private int age;
	private double weight;
	private double height;
	private Gender gender;
	private Activity activity;

	public Person() 
	{
		this("", 0, 0.0, 0.0, Gender.FEMALE, Activity.LOW);
	}

	public Person(String name, int age, double weight, double height, Gender gender, Activity activity) 
	{
		this.name = name;
		this.age = age;
		this.weight = weight;
		this.height = height;
		this.gender = gender;
		this.activity = activity;
	}
    
	public Person(Person other) 
	{
		this(other.name, other.age, other.weight, other.height, other.gender, other.activity);
	}

	public String getName()
	{
		return name;
	}
    
	public void setName(String name)
	{
		this.name = name;
	}

	public int getAge()
	{
		return age;
	}

	public void setAge(int age)
	{
		this.age = age;
	}

	public double getWeight()
	{
		return weight;
	}

	public void setWeight(double weight)
	{
		this.weight = weight;
	}

	public double getHeight()
	{
		return height;
	}

	public void setHeight(double height)
	{
		this.height = height;
	}

	public Gender getGender()
	{
		return gender;
	}

	public void setGender(Gender gender)
	{
		this.gender = gender;
	}

	public Activity getActivity()
	{
		return activity;
	}

	public void setActivity(Activity activity)
	{
		this.activity = activity;
	}

	@Override
	public String toString()                                        // Object.toString() overriding
	{
		return String.format("%s,%d,%.2f,%.2f,%s,%s", name, age, weight, height, gender.toString(), activity.toString());
	}
        
	@Override
	public boolean equals(Object other)                             // Object.equals overriding
	{
		if (other instanceof Person) {
			Person that = (Person) other;
			return that.canEqual(this) 
					&& this.getName().equals(that.getName()) 
					&& this.getAge() == that.getAge() 
					&& this.getWeight() == that.getWeight()
					&& this.getHeight() == that.getHeight()
					&& this.getGender() == that.getGender()
					&& this.getActivity() == that.getActivity();
		}
		return false;
	}
    
	@Override 
	public int hashCode() 
	{
		return (41 * getName().hashCode() + getAge() + Double.valueOf(getWeight()).hashCode() + Double.valueOf(getHeight()).hashCode() + (int)(getGender()).hashCode() + (int)(getActivity()).hashCode());
	}

	public boolean canEqual(Object other) 
	{
		return (other instanceof Person);
	}

	public int compareTo(Person other) 
	{
		return this.getName().compareTo(other.getName()); //ascending order
	}

	public static Comparator<Person> AgeComparator
							= new Comparator<Person>() {
		public int compare(Person p1, Person p2) {	      
			return p1.getAge() - p2.getAge(); //ascending order
		}
	};

	public static Comparator<Person> WeightComparator
							= new Comparator<Person>() {
		public int compare(Person p1, Person p2) {     
			return Double.compare(p1.getWeight(), p2.getWeight()); //ascending order
		}
	};

	public static Comparator<Person> HeightComparator
							= new Comparator<Person>() {
		public int compare(Person p1, Person p2) {	      
			return Double.compare(p1.getHeight(), p2.getHeight()); //ascending order
		}
	};

	public static Comparator<Person> GenderComparator
							= new Comparator<Person>() {
		public int compare(Person p1, Person p2) {	      
			return p1.getGender().compareTo(p2.getGender());
		}
	};

	public static Comparator<Person> ActivityComparator
							= new Comparator<Person>() {
		public int compare(Person p1, Person p2) {	      
			return p1.getActivity().compareTo(p2.getActivity());
		}
	};
}
